# Remote Jupyter in Kubernetes

## Installation

On the local notebook server install the repo:

```
pip install --user git+https://gitlab.eidf.ac.uk/akrause/jupyter-remote.git
```

## Kubeconfig

The kubeconfig for the remote Kubernetes cluster must be available in the user space. Default path is `$HOME/.kube/config`. Jupyterlab doesn't seem to like paths starting with `.` when creating a new directory in the GUI so you'll have to create the `.kube` directory in a terminal.

## Load extension

```
%load_ext jupyter-remote
```

The cell magic `kube_exec` is now available. To list available commands and help for the cell magic:

```
?kube_exec
```

## Commands and options

### `%kube_exec launch`

Launch a new pod in the Kubernetes cluster.

* `-ns/--namespace`: Kubernetes namespace
* `-i/--image`: Container image (required), for example `quay.io/jupyter/pytorch-notebook:cuda12-latest`
* `-k/--kubeconfig`: Path to the kubeconfig (default is `.kube/config`)
* `-pp/--pod-prefix`: Job and pod name prefix (a unique id is appended, default is '')
* `-g/--gpus`: Number of GPUs (default is 1)
* `-gp/--gpu-product`: GPU selector, currently available:
  * `NVIDIA-A100-SXM4-40GB-MIG-1g.5gb` (default)
  * `NVIDIA-A100-SXM4-40GB-MIG-3g.20gb`
  * `NVIDIA-A100-SXM4-40GB`
  * `NVIDIA-A100-SXM4-80GB`
  * `NVIDIA-H100-80GB-HBM3`

GPUs in high demand may have a longer wait time and the request may time out.

### `%kube_exec connect`

Connect to an existing pod.

* `-p/--pod-name`: Name of a running pod to connect to
* `-ns/--namespace`: Kubernetes namespace
* `-k/--kubeconfig`: Path to the kubeconfig

### `%kube_exec jobs`

List jobs in the namespace.

* `-ns/--namespace`: Kubernetes namespace
* `-k/--kubeconfig`: Path to the kubeconfig (default `.kube/config`)

### `%kube_exec pods`

List pods in the namespace.

* `-ns/--namespace`: Kubernetes namespace
* `-k/--kubeconfig`: Path to the kubeconfig (default `.kube/config`)

### `%kube_exec destroy`

Deletes the current job or a named job in the namespace.

* `-ns/--namespace`: Kubernetes namespace (only required if `-p` is specified)
* `-p/--pod-name`: Name of the pod to delete (optional, default is the currently connected container)

### `%%kube_exec`

Execute the cell in the remote Kubernetes Jupyter pod.

## Launch container

Launch a new pod: After launch a new Jupyter kernel is created in the container and utilities required for remote execution are transferred to the pod. The image for the pod must run Jupyter at launch. This could be a Jupyter Docker stack image (see https://jupyter-docker-stacks.readthedocs.io/en/latest/using/selecting.html) or an extension of those.

Example with Pytorch CUDA 12 in namespace eidfxxxns on the GPU Service and pod name prefix `amy-`. By default, it requests one GPU of type `NVIDIA-A100-SXM4-40GB-MIG-1g.5gb`.

```
%kube_exec launch -k /path/to/kube/config -ns eidfxxxns -pp amy- -i quay.io/jupyter/pytorch-notebook:cuda12-latest
```

## Re-use container

Connect to an existing container. A Jupyter server must be running on this container.

```
%kube_exec connect -ns eidfxxxns -p amy-test
```