import json
from pathlib import Path
import requests
import sys
import time
from websocket import create_connection, WebSocketTimeoutException

from jupyter_server.serverapp import list_running_servers

def setup(max_wait=20):
    # wait for server to be available
    i = 0
    servers = list(list_running_servers())
    while not servers:
        time.sleep(1)
        servers = list(list_running_servers())
        if i > max_wait:
            raise('Timeout: Jupyter is not running.')

    # get token and url
    jupyter_config = servers[0]
    
    base = jupyter_config['url']
    token = jupyter_config['token']
    headers = {'Authorization': f'Token {token}'}
    return base, headers

def create_kernel(base, headers):
    url = f'{base}/api/kernels'
    response = requests.post(url,headers=headers)
    kernel = json.loads(response.text)
    return kernel

if __name__ == '__main__':
    base, headers = setup()
    kernel = create_kernel(base, headers)
    try:
        # store the kernel id on disk if requested
        output_file = sys.argv[1]
        with open(output_file, 'w') as f:
            f.write(kernel['id'])
    except:
        pass
    print(kernel['id'])
    print(base)
    print(headers['Authorization'])
    # print(
    #     json.dumps({
    #         'kernel_id': kernel['id'],
    #         'base': base,
    #         'headers': headers,
    #     }),
    #     end=''
    # )
    # print(kernel['id'], end='')
