import datetime
import json
import sys
from urllib.parse import urlsplit
import uuid
from websocket import create_connection, WebSocketTimeoutException, WebSocketBadStatusException

from jupyter_server.serverapp import list_running_servers

def send_execute_request(code, session):
    msg_type = 'execute_request';
    content = { 'code' : code, 'silent': False }
    hdr = {
        'msg_id' : uuid.uuid1().hex, 
        'username': 'test', 
        'session': session, 
        'data': datetime.datetime.now().isoformat(),
        'msg_type': msg_type,
        'version' : '5.0',
    }
    msg = {
        'header': hdr,
        'parent_header': hdr, 
        'metadata': {},
        'content': content,
    }
    return msg

def connect(base, kernel_id, headers):
    u = urlsplit(base)
    return create_connection(f"ws://{u.hostname}:{u.port}/api/kernels/"+kernel_id+"/channels", header=headers)

def execute(code, ws, session):
    ws.send(json.dumps(send_execute_request(code, session)))
    msg_type = None
    execute_input = False
    while True:
        try:
            rsp = json.loads(ws.recv())
            msg_type = rsp["msg_type"]
            if msg_type == 'execute_input':
                execute_input = True
            elif msg_type == 'stream':
                print(json.dumps(rsp))
            elif msg_type == 'execute_result':
                print(json.dumps(rsp))
            elif msg_type == 'display_data':
                print(json.dumps(rsp))
            elif msg_type == 'status' and execute_input and rsp['content']["execution_state"] == "idle":
                break
            elif msg_type == "error":
                print(json.dumps(rsp))
                break
        except WebSocketTimeoutException as _e:
            raise Exception('Websocket timeout')

if __name__ == '__main__':
    kernel_id = sys.argv[1]
    # print('Enter expression:', flush=True)
    codejson = sys.stdin.readline()
    code = json.loads(codejson)['code']
    base = sys.argv[2]
    headers = {'Authorization': sys.argv[3]}
    session = sys.argv[4]
    try:
        ws = connect(base, kernel_id, headers)
    except WebSocketTimeoutException:
        raise
    except WebSocketBadStatusException as exc:
        # # handle 404 if the kernel doesn't exist
        if exc.status_code == 404:
            raise Exception(f'No kernel with id "{kernel_id}"')
        else:
            raise
    except Exception as exc:
        raise
    execute(code, ws, session)
